package com.chelvc.framework.redis.cache;

import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.group.GroupStore;
import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.redis.context.RedisHashHolder;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * 场景分组存储Redis缓存实现
 *
 * @author Woody
 * @date 2024/11/28
 */
public class RedisGroupStore implements GroupStore {
    private final GroupStore delegate;

    public RedisGroupStore(@NonNull GroupStore delegate) {
        this.delegate = delegate;
    }

    /**
     * 获取Redis用户分组标识
     *
     * @return Redis标识
     */
    private String key() {
        Long id = SessionContextHolder.getId();
        return id == null ? null : ("group:" + id);
    }

    @Override
    public Caps get(@NonNull String scene) {
        String key = this.key();
        if (StringUtils.isEmpty(key)) {
            return null;
        }

        // 获取分组标识，如果缓存不存在则重新加载分组并更新缓存
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        Caps group = (Caps) template.opsForHash().get(key, scene);
        if (group == null && (group = this.delegate.get(scene)) != null) {
            RedisHashHolder.set(template, key, scene, group, Duration.ofDays(1));
        }
        return group;
    }

    @Override
    public Map<String, Caps> get(@NonNull Collection<String> scenes) {
        String key = this.key();
        if (StringUtils.isEmpty(key) || ObjectUtils.isEmpty(scenes)) {
            return Collections.emptyMap();
        }

        // 批量获取缓存分组映射
        List<String> nones = Lists.newLinkedList();
        Map<String, Caps> mapping = Maps.newHashMapWithExpectedSize(scenes.size());
        RedisTemplate<String, Object> template = RedisContextHolder.getDefaultTemplate();
        HashOperations<String, String, Caps> operations = template.opsForHash();
        List<Caps> groups = operations.multiGet(key, scenes);
        int i = 0;
        for (String scene : scenes) {
            Caps group = ObjectUtils.isEmpty(groups) ? null : groups.get(i++);
            if (group == null) {
                nones.add(scene);
            } else {
                mapping.put(scene, group);
            }
        }

        // 重新加载场景分组为null的数据
        if (ObjectUtils.notEmpty(nones)) {
            Map<String, Caps> reloads = this.delegate.get(nones);
            if (ObjectUtils.notEmpty(reloads)) {
                mapping.putAll(reloads);
                RedisHashHolder.set(template, key, reloads, Duration.ofDays(1));
            }
        }
        return mapping;
    }

    @Override
    public Caps set(@NonNull String scene, @NonNull Caps group) {
        String key = this.key();
        if (StringUtils.isEmpty(key)) {
            return group;
        }

        // 重置场景分组并更新缓存
        group = this.delegate.set(scene, group);
        RedisHashHolder.set(RedisContextHolder.getDefaultTemplate(), key, scene, group, Duration.ofDays(1));
        return group;
    }

    @Override
    public Map<String, Caps> set(@NonNull Map<String, Caps> groups) {
        String key = this.key();
        if (StringUtils.isEmpty(key)) {
            return groups;
        }

        // 重置场景分组并更新缓存
        groups = this.delegate.set(groups);
        RedisHashHolder.set(RedisContextHolder.getDefaultTemplate(), key, groups, Duration.ofDays(1));
        return groups;
    }
}
